"""Report.py - HTML report generation the cultaptation engine.

Copyright (c) 2008 by Eckhart Arnold (www.eckhartarnold.de)
                   
License: GNU Public License V3
(http://www.gnu.de/documents/gpl-3.0.en.html)

This program is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the 
Free Software Foundation, either version 3 of the License, or (at your 
option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program. If not, see <http://www.gnu.org/licenses/>.
"""


import os.path
from numpy import arange
import matplotlib
matplotlib.use("Agg")
import pylab

STYLE_SHEET = """
table {border-width:1px; border-style:solid; border-color:grey; border-collapse:collapse; }
td {border-width:1px; border-style:solid; border-color:grey;  }
"""

class HTMLPage(object):
    """This class helps producing an HTML page. Different elements like
    headings, images, links can be added with the respective methods.
    """
    def __init__(self, directory, name, pagetitle):
        """Creates a new HTMLPage object, where 'directory' is the directory
        where the page will be stored, 'name' is its name (without the ".html"
        suffix!) and 'pagetitle' a suitable title for the page.
        """
        self.directory = directory
        self.name = name
        self.head = ["<html>\n<head>\n<title>"+pagetitle+"</title>\n"+\
            '<link rel="stylesheet" type="text/css" href="stylesheet.css"'+\
            '<body>\n<a name="top"></a>\n']
        self.tail = ["</body","</html>"]
        self.toc = []
        self.tocCounter = [0, 0, 0, 0, 0, 0]
        self.chunks = []
        self.plotCounter = 0
        self.postscript = False
        self.back='<a href="#top" style="text-align:right;">--> back</a><br />'        
    
    def setPostscript(self, onOff):
        """Turns the creation of additional postscript graphics on or off."""
        self.postscript = onOff
    
    def text(self, text):
        """Adds some 'text' to the page. The text may contain 
        any html tags."""
        self.chunks.append(text)
        
    def crlf(self):
        """Adds a linefeed."""
        self.chunks.append("<br />")
    
    def heading(self, text, level = 1, tocEntry = True):
        """Adds the heading 'text' with a 'level' from 1-6 (1=biggest, 
        6=smallest) to the HTML page. If 'tocEntry' is 'True', the
        heading will also be added to the table of contents at the
        beginning of the page."""
        openingTag = ["<br /><h1>", "<br /><h2>", "<br /><h3>", "<br /><h4>", "<br /><h5>", "<br /><h6>"][level-1]
        closingTag = ["</h1>","</h2>","</h3>","</h4>","</h5>","</h6>"][level-1]
        if tocEntry:
            ref = "toc"+str(len(self.toc))
            if self.tocCounter[level-1] > 0:
                self.chunks.append(self.back)
            self.tocCounter[level-1] += 1
            numStr = ".".join([str(i) for i in self.tocCounter[0:level] if i != 0])+". ";
            prefix = "&nbsp;"*2*level+numStr;
            self.toc.append(prefix+'<a href="#'+ref+'">'+text+'</a><br />')
            self.anchor(ref)            
        else: numStr = "";
        self.chunks.append(openingTag+numStr+text+closingTag)
        
    def anchor(self, name):
        """Adds an anchor for a link to the page."""
        self.chunks.append('<a name="'+name+'"> </a>')
        
    def link(self, ref, text):
        """Adds a to 'ref' to the page with the linktext 'text'."""
        self.chunks.append('<a href="'+ref+'">'+text+'</a>')
        
    def openParagraph(self):
        """Adds an opening tag for a new paragraph."""
        self.chunks.append("<p>")
        
    def closeParagraph(self):
        """Adds an closing tag for a new paragraph."""
        self.chunks.append("</p>")
        
    def openNumberedList(self):
        """Starts a numbered list."""
        self.chunks.append("<ol>")
        
    def listItem(self, text):
        """Adds an item to the list."""
        self.chunks.append("<li>"+text+"</li>")
        
    def closeNumberedList(self):
        """Ends a numbered list."""
        self.chunks.append("</ol>")

    def image(self, relativeFileName):
        """Adds the image with the 'relativeFileName'."""
        self.chunks.append('<img src="'+relativeFileName+\
                           '" alt="'+relativeFileName+'" />')                
        
    def _doPlot(self, plotfunc, xlabel, ylabel, title, postscript):
        """Internal plot function. The actual plot is done via callback
        of 'plotfunc'."""
        self.plotCounter += 1
        fileName = os.path.join("images", self.name+str(self.plotCounter))
        path = os.path.join(self.directory, fileName)
        pylab.clf()
        #matplotlib.use("Agg")
        plotfunc()
        if xlabel: pylab.xlabel(xlabel)
        if ylabel: pylab.ylabel(ylabel)
        if title: pylab.title(title)
        pylab.savefig(path+".png")
        self.image(fileName+".png")
        if postscript:
            #matplotlib.use("PS")
            pylab.savefig(path+".ps")
        pylab.clf()
        
    def plot(self, names, data, xlabel="", ylabel="", title=""):
        """Plots the 'data' array to an image file on the disk and
        adds the image to the HTML page. 'data' is a two dimensional array
        with one row for each name in the list 'names'. If 'postscript'
        is 'True' than a postscript version of the plot will be created as
        well and a link will be added to download the postscript version."""
        def callback():
            for i in xrange(len(names)): pylab.plot(data[i])            
            pylab.legend(tuple(names), loc="upper left")
        self._doPlot(callback, xlabel, ylabel, title, self.postscript)
        
    def singlePlot(self, data, xlabel="", ylabel="", title=""):
        """Plots the 'data' array to an image file on the disk and
        adds the image to the HTML page. 'data' is a one dimensional array
        of y values. If 'postscript' is 'True' than a postscript version of 
        the plot will be created as well and a link will be added to 
        download the postscript version."""
        def callback():
            pylab.plot(data)            
        self._doPlot(callback, xlabel, ylabel, title, self.postscript)        
        
    def barChart(self, data, xlabel="", ylabel="", title=""):
        """Plots the one dimensional array 'data' as a bar chart,
        stores the image in a file and adds it to the HTML page."""
        def callback():
            pylab.bar(arange(len(data)), data, width=1.0)
        self._doPlot(callback, xlabel, ylabel, title, self.postscript)            
         
    def table(self, data):
        """Adds the two dimensional array 'data' as a table to the
        HTML-Page. 'data' may contain strings, integers, floats or
        arbitrary objects that can be converted to strings."""
        self.chunks.append("<table>")
        for r in xrange(data.shape[0]):
            self.chunks.append("<tr>")
            for c in xrange(data.shape[1]):
                row = []; row.append("<td>")
                if str(type(data[r,c])).find("float") >= 0:
                    item = "%1.2f"%data[r,c]
                else: item = str(data[r,c])
                row.append(item)
                row.append("</td>")
                self.chunks.append(" ".join(row))
            self.chunks.append("</tr>")
        self.chunks.append("</table>")
        
    def _createStylesheet(self):
        """Create the style sheet file for the HTML page if it does not exist
        already."""
        global STYLE_SHEET
        fileName = os.path.join(self.directory, "stylesheet.css")
        f = file(fileName, "w")
        f.write(STYLE_SHEET)
        f.close()
        
    def saveToDisk(self):
        """Finally, saves the HTML page. Returns the filename or
        an empty string if an error occured."""
        assert len(self.chunks) >= 1, "Tried to write empty HTML page."
        path = os.path.join(self.directory, self.name+".html")
        try:
            self._createStylesheet()
            f = file(path, "w")
            if self.toc:  self.toc.insert(0, "<h1>Table of contents</h1>")
            if self.chunks[-1] != self.back: self.chunks.append(self.back)
            page = "\n".join(self.head) + "\n".join(self.toc) + \
                   "\n".join(self.chunks) + "\n".join(self.tail)
            f.write(page)
            f.close()
            return path
        except IOError:
            print 'IOError while writing file "%s": %s %s' % \
                  (IOError.filename, IOError.message, IOError.errno)
            return ""
