"""cultaptation engine

Copyright (c) 2008 by Eckhart Arnold (www.eckhartarnold.de)
                   
License: GNU Public License V3
(http://www.gnu.de/documents/gpl-3.0.en.html)

This program is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the 
Free Software Foundation, either version 3 of the License, or (at your 
option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from Engine_optimized import *

def PairwiseMatch(defender, invader):
    """Plays one match between a 'defender' strategy that is allowed
    to dominate the population during the first Configuration.PW_INIT_PHASE
    number of round and an 'invader' that may invade the population through
    mutation after this initial phase.
    Returns a dictionary that contains the average frequencies for
    'defender' and 'invader' during the last quarter of the match.
    """
    resetLastRoundExploiters()
    
    Nobserve = PW_NOBSERVE
    obsMistakes = PW_OBS_MISTAKES
    obsDeviation = PW_OBS_DEVIATION
    pc = PW_PC
    
    strategies = [defender, invader]
    environment = createEnvironment()
    population = array([Individual(defender) for i in xrange(POPULATION_SIZE)])
    Monitor.PWStart(environment, strategies, population)
    
    for i in xrange(PW_INIT_PHASE):
        update = OneRound(strategies, population, environment, 0.0,
                          pc, Nobserve, obsMistakes, obsDeviation)
        population, environment = update
        Monitor.PWRecord(population)
    
    for i in xrange(PW_INIT_PHASE, MATCH_LENGTH*3/4):
        update = OneRound(strategies, population, environment, MUTATION_RATE, 
                          pc, Nobserve, obsMistakes, obsDeviation)
        population, environment = update
        Monitor.PWRecord(population)        
    
    frequencies = {}.fromkeys(strategies, 0)
    for i in xrange(MATCH_LENGTH*3/4, MATCH_LENGTH):
        update = OneRound(strategies, population, environment, MUTATION_RATE, 
                          pc, Nobserve, obsMistakes, obsDeviation)
        population, environment = update
        for individual in population: frequencies[individual.strategy] += 1 
        Monitor.PWRecord(population)
                
    for strategy in frequencies.keys(): 
        frequencies[strategy] /= float(MATCH_LENGTH/4)
        
    return frequencies


def MeleeMatch(strategies):
    """Plays one melee match (everybody against everybody) between the given
    'strategies'. Returns a dictionary with the average frequencies of the 
    strategies during the last quarter of the match. 
    (Besides the 'strategies' given the returned dictionary also contains 
    one entry for the "stuffing-strategy" of which the population is composed
    that the competing strategies invade during the melee match. Theoretically,
    the average population of the "stuffing-strategy" may still be greater 
    than zero during the last quarter of the match.) 
    """        
    resetLastRoundExploiters()

    environment = createEnvironment()
    population = array([Individual(Stuffing) for i in xrange(POPULATION_SIZE)])
    Monitor.MMStart(environment, strategies+[Stuffing], population)

    for i in xrange(MATCH_LENGTH*3/4):
        pc = pcvariate(PC_MIN, PC_MAX) 
        Nobserve = randint(N_OBSERVE_MIN, N_OBSERVE_MAX)
        obsMistakes = uniform(OBS_MISTAKES_MIN, OBS_MISTAKES_MAX) # OM_MAX not included!
        obsDeviation = uniform(OBS_DEVIATION_MIN, OBS_DEVIATION_MAX) # OD_MAX not included!
        update = OneRound(strategies, population, environment, MUTATION_RATE,  
                          pc, Nobserve, obsMistakes, obsDeviation)
        population, environment = update
        Monitor.MMRecord(population)
    
    frequencies = {}.fromkeys(strategies+[Stuffing], 0)    
    for i in xrange(MATCH_LENGTH*3/4, MATCH_LENGTH):
        pc = pcvariate(PC_MIN, PC_MAX)
        Nobserve = randint(N_OBSERVE_MIN, N_OBSERVE_MAX)
        obsMistakes = uniform(OBS_MISTAKES_MIN, OBS_MISTAKES_MAX) # OM_MAX not included!
        obsDeviation = uniform(OBS_DEVIATION_MIN, OBS_DEVIATION_MAX) # OD_MAX not included!
        update = OneRound(strategies, population, environment, 0.0,
                          pc, Nobserve, obsMistakes, obsDeviation)
        population, environment = update
        for individual in population: frequencies[individual.strategy] += 1 
        Monitor.MMRecord(population)
                        
    for strategy in frequencies.keys():
        frequencies[strategy] /= float(MATCH_LENGTH/4)
        
    return frequencies
        


def PairwiseContest(strategyA, strategyB):
    """Runs a whole contest of pairwise matches between 'strategyA' and 
    'strategyB', i.e. a number of Configuration.PW_REPETITIONS / 2 matches
    where 'strategyA' is the defender and 'strategyB' the invader and
    the same number of matches with swapped roles.
    Returns a dictionary with the average frequencies of 'strategyA' and
    'strategyB' during the last quarter of all of the matches.
    """
    assert PW_REPETITIONS % 2 == 0, "Need an even number of repetitions!"
    fA, fB = 0.0, 0.0
    Monitor.PCStart(strategyA, strategyB)
    for i in xrange(PW_REPETITIONS/2):
        result = PairwiseMatch(strategyB, strategyA)
        fA += result[strategyA]
        fB += result[strategyB]
        Monitor.PCRecord(result)
    freqA, freqB = fA, fB
    Monitor.PCSwap({strategyA: fA/(PW_REPETITIONS/2), 
                    strategyB: fB/(PW_REPETITIONS/2)})
    fA, fB = 0.0, 0.0
    for i in xrange(PW_REPETITIONS/2):
        result = PairwiseMatch(strategyA, strategyB)
        fA += result[strategyA]
        fB += result[strategyB]
        Monitor.PCRecord(result)
    freqA += fA;  freqB += fB
    Monitor.PCSwap({strategyA: fA/(PW_REPETITIONS/2), 
                    strategyB: fB/(PW_REPETITIONS/2)})
    freqA /= PW_REPETITIONS
    freqB /= PW_REPETITIONS
    return {strategyA: freqA, strategyB: freqB}



def MeleeContest(strategies):
    """Runs a melee contest, where all given 'strategies' are matched in
    a numaber of Configuration.MELEE_REPETITIONS matches. Returns a 
    dictionary that is indexed by the strategies and contains their
    average frequencies during the last quarter of all matches.
    """
    frequencies = {}.fromkeys(strategies+[Stuffing], 0)
    Monitor.MCStart(strategies+[Stuffing])
    for i in xrange(MELEE_REPETITIONS):
        result = MeleeMatch(strategies)
        for s,f in result.items(): frequencies[s] += f
        Monitor.MCRecord(result)
    for s in frequencies.keys(): 
        frequencies[s] /= MELEE_REPETITIONS
    return frequencies     


def FullTournament(strategies):
    """Runs a full tournament with both stages, i.e. stage 1: pairwises
    matches of every strategy against every other strategy; stage 2:
    melee matches of the ten best strategies from the first stage.
    """
    wonContests = {}.fromkeys(strategies, 0.0)
    Monitor.TournamentStart(strategies)
    for i in xrange(len(strategies)):
        for k in xrange(i+1, len(strategies)):
            s1 = strategies[i]
            s2 = strategies[k]
            Monitor.TournamentPhase1(s1, s2)
            result = PairwiseContest(s1, s2)
            Monitor.TournamentPhase1Record(result)
            if result[s1] > result[s2]:   wonContests[s1] += 1.0
            elif result[s2] > result[s1]: wonContests[s2] += 1.0
            else:
                wonContests[s1] += 0.5
                wonContests[s2] += 0.5
    ranking = [(it[1],it[0]) for it in wonContests.iteritems()]
    ranking.sort(reverse=True)
    winners = [ranking[i][1] for i in range(min(10, len(ranking)))]
    Monitor.TournamentPhase2(winners, ranking)
    result = MeleeContest(strategies)
    return result

        
if __name__ == "__main__":
    result = MeleeMatch([ExampleStrategy, SimpleObserver, ASocial])
    # result = PairwiseMatch(SimpleObserver, ExampleStrategy)
    Monitor.CreateGenericDir()
    Monitor.PCRecord(result)
    if LIVE_PLOT: Monitor.gfx.waitUntilClosed()
