"""cultaptation engine

Copyright (c) 2008 by Eckhart Arnold (www.eckhartarnold.de)
                   
License: GNU Public License V3
(http://www.gnu.de/documents/gpl-3.0.en.html)

This program is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the 
Free Software Foundation, either version 3 of the License, or (at your 
option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from random import random as rnd
from math import sqrt
from random import expovariate, choice, randint, gauss, uniform
from numpy import array, arange, zeros, where, concatenate, setxor1d, int16

from Configuration import *
from Strategy import *
from Toolkit import *
import Monitor



def GenPayoffValue():
    """Generates a new payoff value from a distribution with many relatively
    small payoffs and some large payoffs. Payoffs may be zero but not smaller.
    """
    return int(min(expovariate(1.0/10.0), MAX_PAYOFF-MIN_PAYOFF)+MIN_PAYOFF)
    # return exponential(10, (DISTRIBUTION_SIZE,)).astype(int16)+1 
    

def createEnvironment():
    """Creates an environment, i.e. a 1-dimensional array of payoff values, 
    and fills it with payoff values taken from the distribution
    """
    return array([GenPayoffValue() for i in xrange(ENVIRONMENT_SIZE)], dtype = int16)



def changeEnvironment(environment, pc):
    """Changes each entry of the 'environment' array with probability 'pc'.
    'environment' is changed in place!)
    """
    for i in xrange(len(environment)):
        if rnd() < pc:
            environment[i] = GenPayoffValue()



class Individual(object):
    """Defines an individual. The defining characteristics of an individual
    are its strategy, repertoire, history, age and collected payoffs.
    """
    def __init__(self, strategy):
        self.strategy = strategy
        self.history = zeros((4,0), dtype=int16)
        self.repertoire = zeros((2,0), dtype=int16)
        self.roundsAlive = 0
        self.aggregatedPayoff = 0
        
    def meanPayoff(self):
        """Returns the mean payoff the individual gained
        so far.
        """
        if self.roundsAlive == 0: return 0
        return float(self.aggregatedPayoff) / self.roundsAlive   
    def getHistory(self):
        return self.history
    def getRepertoire(self):
        return self.repertoire

           
def EvolutionaryDynamics(strategies, population, mutationRate):
    """Determines the next round's population composition
    depending on this generation's population, the death probability
    and mutation rate and the available strategies.
    Returns the next generation's population.
    """ 
    sigma = sum([individual.meanPayoff() for individual in population])
    if sigma == 0:  
        pr = [1.0 / len(population)]*len(population)
    else: 
        pr = [individual.meanPayoff() / sigma for individual in population]
    next_generation = population.copy()
    for i in xrange(len(population)):
        if rnd() < DEATH_PROBABILITY:
            if rnd() < mutationRate:
                others = strategies[:]
                try: others.remove(population[i].strategy)
                except ValueError: pass
                if others: parent = choice(others)
                else: parent = population[i].strategy
            else:
                parent = population[RandomSelect(pr)].strategy
            Monitor.Death(population[i])
            next_generation[i] = Individual(parent)
    return next_generation



lastRoundExploiters = {}

def resetLastRoundExploiters():
    global lastRoundExploiters
    lastRoundExploiters = {}


def OneRound(strategies, population, environment, mutationRate,
             pc, Nobserve, obsMistakes, obsDeviation):
    """Computes one round of the evolutionary simulation. Returns a
    tuple (population, environment) with the population for the next
    round and the (possibly changed) environment.
    """
    global lastRoundExploiters
    exploiters = {}; observers = []
    
    for individual in population:
        move, repertoire = individual.strategy(individual.roundsAlive,
                                               individual.repertoire.copy(),
                                               individual.history)
        assert (repertoire[0,:] == individual.repertoire[0,:]).all(), \
            "Illegal manipulation of repertoire by strategy: " + \
            individual.strategy.func_name+" !"
        if move == OBSERVE: observers.append(individual)
        else:
            if move == INNOVATE:
                possible_innovations = setxor1d(arange(EXPLOIT_MIN, EXPLOIT_MAX+1, dtype=int16),
                                                repertoire[0,:])
                if len(possible_innovations) > 0:
                    innovation = choice(possible_innovations)
                    payoff = environment[innovation-1]
                    new_entry = array([[innovation],[payoff]], dtype=int16)
                    repertoire = concatenate((repertoire, new_entry),1)
                else:
                    innovation = 0
                    payoff = 0
                new_record = array([[individual.roundsAlive+1],
                                    [INNOVATE], [innovation], [payoff]], dtype=int16)
            else: # move == EXPLOIT
                assert move in repertoire[0,:], "Move "+str(move)+\
                    " by strategy "+individual.strategy.func_name+\
                    " is illegal, because it is not in the repertoire!\n\n"+\
                    "Repertoire:\n"+str(repertoire) + "\n\n" +\
                    "History:\n"+str(individual.history)+"\n\n"
                payoff = environment[move-1] 
                individual.aggregatedPayoff += payoff
                repertoire[1, where(repertoire[0,:]==move)] = payoff
                new_record = array([[individual.roundsAlive+1],
                                    [move], [move], [payoff]], dtype=int16)
                exploiters[individual] = (move, payoff)
            individual.roundsAlive += 1
            individual.repertoire = repertoire
            individual.history = concatenate((individual.history, new_record),1)
            
    for individual in observers:
        if OBSERVATION_DELAY:  observables = lastRoundExploiters
        else: observables = exploiters
        others = observables.keys()
        new_record = zeros((4, Nobserve), dtype = int16)
        i = 0
        repertoire = individual.repertoire
        while i < Nobserve and others:
            spied = choice(others)
            others.remove(spied)
            act, payoff = observables[spied]
            if rnd() < obsMistakes: 
                act = randint(EXPLOIT_MIN, EXPLOIT_MAX)
            payoff = max(0, int(gauss(payoff, obsDeviation)+0.5))
            new_record[:,i] = [individual.roundsAlive+1, OBSERVE, act, payoff]
            if act in repertoire[0,:]:
                repertoire[1, where(repertoire[0,:]==act)] = payoff
            else:
                new_entry = array([[act],[payoff]], dtype = int16)
                repertoire = concatenate((repertoire, new_entry), 1)
            i += 1
        while i < Nobserve:
            new_record[:,i] = [individual.roundsAlive+1, OBSERVE, 0, 0]
            i += 1 
        individual.roundsAlive += 1
        individual.repertoire = repertoire
        individual.history = concatenate((individual.history, new_record),1)
        
    lastRoundExploiters = exploiters
    population = EvolutionaryDynamics(strategies, population, mutationRate)
    changeEnvironment(environment, pc)
    return (population, environment)



