"""Monitory.py - monitoring module for the cultaptation engine
This modue collects and assembles statistical data about the
matches and the strategies.

Copyright (c) 2008 by Eckhart Arnold (www.eckhartarnold.de)
                   
License: GNU Public License V3
(http://www.gnu.de/documents/gpl-3.0.en.html)

This program is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the 
Free Software Foundation, either version 3 of the License, or (at your 
option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from random import random as rnd
import time, os.path, math
from numpy import array, arange, zeros, mean, amax, where, int8, int16, float32

import Configuration as CFG
from Configuration import INNOVATE, OBSERVE, ILLEGAL, EXPLOIT_MIN, EXPLOIT_MAX
import Toolkit, Report

gfx = None
try:
    from PyPlotter import Gfx, Graph, Compatibility
    HAS_PYPLOTTER = True    
except ImportError:
    HAS_PYPLOTTER = False
    

# global variables

base_counter = 0
match_counter = 0
round_counter = 0
data = None
contest_data = None
tournament_data = None
matchCollection = []
output_path = ""
tournament_path, phase1_path, phase2_path = "","",""

progressCallback = lambda matchNr, roundNr: None    
rlgCallback = lambda : None

def resetLiveGraph():
    global liveGraph, optFilter
    if CFG.LIVE_PLOT and HAS_PYPLOTTER:
        liveGraph = Graph.Cartesian(Gfx.nilDriver(), 0, 0, CFG.MATCH_LENGTH, 100, 
                            "Live Plot: Frequencies", "round", "frequency", 
                            styleFlags = Graph.DEFAULT_STYLE^Graph.TITLE,
                            region = (0.0, 0.0, 1.0, 1.0))
        optFilter = Graph.OptimizingFilter(liveGraph)
        rlgCallback()
resetLiveGraph()
    
    
def LivePlot(frequencies, round):
    """Plots of the strategies' frequencies (and payoffs) while a match
    is running."""
    global liveGraph, optFilter, driver, gfx
    if CFG.LIVE_PLOT and HAS_PYPLOTTER:
        if isinstance(liveGraph.gfx, Gfx.nilDriver):
            if not gfx:
                driver = Compatibility.GetDriver()
                gfx = driver.Window((1024, 680), "cultaptation - Live Plot")
            liveGraph.changeGfx(gfx)
        for (key, value) in frequencies.items():
            optFilter.addValue(key.func_name, round, value)
        if round %10 == 0: liveGraph.gfx.refresh()


example_counter = 0 # not the best style, but...
class PostMortemInfo(object):
    """The PostMortemInfo class collects some statistical information about am 
    individual after it has deceased:
    
        strategy_name - string: the name of the individual's strategy
        strategy_doc - doc string of the individual's strategy
        samples     - number of samples
        lifetime    - the lifetime of the individual
        deathround  - the round when the individual died
        exploits    - the number of EXPLOIT acts
        innovations - the number of INNOVATION acts
        observes    - the number of OBSERVE acts    
        history     - the history record of the individual
        payoffs     - the individual's payoffs for every life round
        avpayoffs   - the individual's average payoffs so far for every life round
        meanPayoff  - the mean payoff of the individual during its lifetime
        maxPayoff   - the maximum payoff the individual ever received
        rSize       - the size of the individual's repertoire at the time of
                      its death
        rMaxPayoff  - the maxmimum expected payoff registered in the repertoire
    """
    def __init__(self, individual, round):
        self.strategy_name = individual.strategy.func_name
        self.strategy_doc = individual.strategy.__doc__
        self.samples = 1
        self.lifetime = individual.roundsAlive
        self.deathround = round
        self.history= individual.getHistory().copy()
        exploits = where(self.history[1,:] >= EXPLOIT_MIN)[0]
        exploitRounds = self.history[0, exploits]-1
        self.payoffs = zeros((self.lifetime,), dtype = int16)        
        self.payoffs[exploitRounds] = self.history[3, exploits]
        self.avpayoffs = zeros((self.lifetime,), dtype = float32)
        for i in xrange(1, self.lifetime):
            self.avpayoffs[i] = float32(sum(self.payoffs[0:i]))/float32(i)
        self.exploits = exploits.shape[0]
        self.innovations = where(self.history[1,:]==INNOVATE)[0].shape[0]
        self.observes = self.lifetime - self.exploits - self.innovations       
        self.meanPayoff = mean(self.payoffs)
        self.maxPayoff = max(self.payoffs)
        
#        invList = []; obsList = []; previousPayoff = 0
#        for i in xrange(exploits.shape[0]):
#            index = exploits[i]
#            delta = self.payoffs[i] - previousPayoff
#            act = individual.history[1, index-1]
#            if act == INNOVATE: invList.append([index-1, delta]) 
#            elif act == OBSERVE: obsList.append([index-1, delta])
#            previousPayoff = self.payoffs[i]
#        self.innovateProfile = array(invList, dtype = int16)
#        self.observeProfile = array(obsList, dtype = int16)
           
        r = individual.getRepertoire()
        self.rSize = r.shape[1]
        self.rMaxPayoff = Toolkit.repMaxPayoff(r)

    def output(self, htmlPage, headingLevel=4):
        """Adds post mortem information about the individual to 
        the Report.HTMLPage object 'htmlPage'.
        """
        global example_counter
        if isinstance(self, PostMortemSummary):
            htmlPage.heading("Average performance profile of strategy: %s"%\
                             self.strategy_name, headingLevel, False)
            htmlPage.openParagraph()
            htmlPage.text("Strategy-Description: <br /><pre>\n" + self.strategy_doc +"\n</pre>\n")
            htmlPage.closeParagraph()            
            payoffsText = "Average Payoffs "
            avPayoffsText = "Average Mean Payoffs"
            example_counter = 0
        else: 
            example_counter += 1
            htmlPage.heading("%i. Example performance profile of strategy: %s"%\
                             (example_counter, self.strategy_name), headingLevel, False)
            payoffsText = "Payoffs "
            avPayoffsText = "Mean Payoffs"
        
        htmlPage.heading("Post Mortem Information:", headingLevel+1, False)      
        data_table = [["Strategy:", self.strategy_name, ""],
                      ["Life Time:", self.lifetime, "rounds"],
                      ["Time of Death:", self.deathround, "round"],
                      ["","",""],
                      ["Repertoire Size:", self.rSize, "entries"],
                      ["Max. Payoff in Rep.", self.rMaxPayoff, "units"],
                      ["","",""],
                      ["Mean Payoff:", self.meanPayoff, "per round"],
                      ["Maximum Payoff:", self.maxPayoff, "per round"],
                      ["","",""],
                      ["EXPLOIT", self.exploits, "times"],
                      ["INNOVATE", self.innovations, "times"],
                      ["OBSERVE", self.observes, "times"]]
        if self.__dict__.has_key("samples"):
            data_table.insert(1, ["Samples:", self.samples, ""])
        htmlPage.table(array(data_table, dtype=object))
        
        htmlPage.heading(payoffsText + "(Graph):", headingLevel+1, False)        
        htmlPage.barChart(self.payoffs, "round", "payoff", 
                payoffsText+"during the life cycle of: %s"%self.strategy_name)
        
        htmlPage.heading(avPayoffsText + "(Graph):", headingLevel + 1, False)
        htmlPage.singlePlot(self.avpayoffs, "round", "mean payoff", 
                payoffsText+"during the life cycle of: %s"%self.strategy_name)
        
        if not isinstance(self, PostMortemSummary): 
            htmlPage.heading("History:", headingLevel+1, False)
            table = zeros((4, self.history.shape[1]+1), dtype = object)
            table[:,0] = ["Round:", "Move:", "Field:", "Payoff"] 
            table[:,1:] = self.history
            for i in xrange(1, self.history.shape[1]+1):
                if table[1, i] == INNOVATE: table[1,i] = "IN"
                elif table[1, i] == OBSERVE: table[1,i] = "OB"
            htmlPage.table(table)
                   
        
        
class PostMortemSummary(PostMortemInfo):
    """The 'PostMortemSummary' object summarizes the information of several
    PostMortemInfo objects. It contains the the attributes as a 'PostMortemInfo'
    object, only that these attributes are filled with the mean values over several
    'PostMortemInfo' objects.
    """
    def __init__(self, pmiList):
        assert len(pmiList) >= 1
        self.strategy_name = pmiList[0].strategy_name
        self.strategy_doc = pmiList[0].strategy_doc
        self.samples = sum([pmi.samples for pmi in pmiList])
        self.lifetime = mean([pmi.lifetime for pmi in pmiList])
        self.deathround = mean([pmi.deathround for pmi in pmiList])
        self.payoffs = self.meanProfile([pmi.payoffs for pmi in pmiList])
        self.avpayoffs = self.meanProfile([pmi.avpayoffs for pmi in pmiList])
        self.exploits = mean([pmi.exploits for pmi in pmiList])
        self.innovations = mean([pmi.innovations for pmi in pmiList])
        self.observes = mean([pmi.observes for pmi in pmiList])
        self.meanPayoff = mean(self.payoffs)
        self.maxPayoff = mean([pmi.maxPayoff for pmi in pmiList])
#        self.innovateProfile = self.meanProfile([pmi.innovateProfile for pmi in pmiList])
#        self.observeProfile = self.meanProfile([pmi.observeProfile for pmi in pmiList])
        self.rSize = mean([pmi.rSize for pmi in pmiList])
        self.rMaxPayoff = mean([pmi.rMaxPayoff for pmi in pmiList])
        
    def meanProfile(self, arrays):
        if not arrays: return array([0])
        maxlen = min(max([len(a) for a in arrays]), 70)
        mp = zeros((maxlen,), dtype = float32)
        for i in xrange(maxlen):
            mp[i] = mean([arrays[k][i] for k in xrange(len(arrays)) \
                         if i < arrays[k].shape[0]])
        return mp

        
def Death(individual):
    """Collect information about an individual after it died in
    case POST_MORTEM_ANALYSIS is enabled."""
    if CFG.POST_MORTEM_ANALYSIS and \
       data.round >= CFG.POST_MORTEM_SEGMENT[0] and \
       data.round <= CFG.POST_MORTEM_SEGMENT[1] and \
       (individual.strategy in CFG.POST_MORTEM_STRATEGIES or \
       (not CFG.POST_MORTEM_STRATEGIES)): # and \
       #rnd() < 1.0 / (1+len(data.forensics[individual.strategy])/100):
        data.forensics[individual.strategy].\
                    append(PostMortemInfo(individual, data.round))


def dumpRanking(htmlPage, result):
    """Writes ranking information for the frequency dictionary 'restult'
    to the given 'htmlPage'. Returns the ranking."""
    ranking = [(result[key], key) for key in result]
    ranking.sort(reverse=True)
    htmlPage.heading("Ranking:", 2, True)
    htmlPage.text('<div style="font-family: monospace;">')
    htmlPage.openNumberedList()
    i = 1
    for freq, strategy in ranking:
        s = list(strategy.func_name)
        s = "".join((s + ["&nbsp;"]*(30-len(s)-int(math.log10(i))))[0:40])
        htmlPage.listItem("%s  %1.4f"% (s, freq))
        i += 1
    htmlPage.closeNumberedList()
    htmlPage.text('</div>')
    return ranking


class MatchData(object):
    """The 'MatchData' class collects the statistical information about
    one match (either pairwise or melee):
    
    strategies  - a list of the strategies
    type        - the type of the match, either "Pairwise" or "Melee"
    matchNr     - the number of the match
    result      - dictionary containing the average frequencies of the 
                  strategies during the last quarter of the match
    description - a string describing the match, 
                  i.e. "Pairwise match nr. 7 of strategies 'Good' and 'Bad'"
    round       - the current round of the match (the final round after
                  the match has ended)                
    frequencies - a 2d-array that records the frequencies of the strategies
                  during the match
    payoffs     - a 2d-array that records the payoffs of the strategies
                  during the match
    mean_payoffs- a 2d-array that records the mean payoffs of the strategies
                  during the match
    forensics   - A dictionary with list of 'PostMortem' objects for each 
                  strategy
                  
                  (some of the attributes are only present if the respective
                  configuration flags are set!)
    ranking     - The ranking of the stratgy as a list of tuples
                  (strategy, average frequency) that is ordered by the 
                  frequency. This field is only valid in after 
                  method 'output' has been called! 
    """
    def __init__(self, strategies, typeString, matchNr):
        self.strategies = strategies
        self.type = typeString
        self.matchNr = matchNr
        self.result = {}
        self.description = typeString + " match nr. " + str(matchNr) + \
                           " of strategies: " + \
                           "".join(["'"+s.func_name+"', " for s in strategies[:-1]]) +\
                           "'" + strategies[-1].func_name + "'"
        self.round = 0
        if CFG.REGISTER_FREQUENCIES:
            self.frequencies = zeros((len(strategies), CFG.MATCH_LENGTH), \
                                     dtype = int8)
        if CFG.REGISTER_PAYOFFS:
            self.payoffs = zeros((len(strategies), CFG.MATCH_LENGTH), \
                                 dtype = float32)
        if CFG.REGISTER_MEAN_PAYOFFS:
            self.mean_payoffs = zeros((len(strategies), CFG.MATCH_LENGTH), \
                                      dtype = float32)
        if CFG.POST_MORTEM_ANALYSIS:
            self.forensics = {}
            for s in strategies: self.forensics[s] = []
        self.ranking = []

    def record(self, population):
        """Records the data from a new round of the match."""
        if CFG.REGISTER_FREQUENCIES or CFG.REGISTER_PAYOFFS or \
           CFG.REGISTER_MEAN_PAYOFFS or CFG.LIVE_PLOT:
            frequencies = {}.fromkeys(self.strategies, 0) 
            for ind in population: frequencies[ind.strategy] += 1
            if CFG.REGISTER_FREQUENCIES:    
                self.frequencies[:, self.round] = \
                  array([frequencies[s] for s in self.strategies],dtype=int8)
        if CFG.REGISTER_PAYOFFS:
            payoffs = {}.fromkeys(self.strategies, 0)            
            for ind in population:
                if ind.roundsAlive >= 1 and ind.history[1, -1] >= EXPLOIT_MIN: 
                    payoffs[ind.strategy] += ind.history[3, -1]
            for key in payoffs.keys():
                f = float(frequencies[key])
                if f > 0.0: payoffs[key] /= f        
            self.payoffs[:, self.round] = \
                array([payoffs[s] for s in self.strategies], dtype = int8)
        if CFG.REGISTER_MEAN_PAYOFFS:
            mean_payoffs = {}.fromkeys(self.strategies, 0)            
            for ind in population: mean_payoffs[ind.strategy] += ind.meanPayoff()
            for key in mean_payoffs.keys():
                f = float(frequencies[key])
                if f > 0.0: mean_payoffs[key] /= f
            data.mean_payoffs[:, self.round] = \
                array([mean_payoffs[s] for s in self.strategies], dtype = int8)
        self.round += 1                          
        if CFG.LIVE_PLOT: LivePlot(frequencies, self.round)
            
    def wrapItUp(self):
        """Reduce the data set of the 'self.forensics' dictionary by making
        a summaray each list and keeping three examples from each list."""
        if CFG.POST_MORTEM_ANALYSIS:
            for strategy in self.forensics.keys():
                pmiList = self.forensics[strategy]
                if len(pmiList) >= 1:
                    # create a summary and keep three examples
                    self.forensics[strategy] = [PostMortemSummary(pmiList)]
                    avLifeTime = int(1.0/CFG.DEATH_PROBABILITY)
                    samples =  [pmi for pmi in pmiList \
                                if pmi.lifetime > avLifeTime/2 and \
                                   pmi.lifetime < avLifeTime*2] # only pick typical examples
                    if len(samples) < 3:
                        self.forensics[strategy].extend(samples)
                    else:
                        self.forensics[strategy].extend([samples[0], # take a sample from the beginning, middle + end.
                            samples[len(samples)/2], samples[-1]]) 
                else: del self.forensics[strategy]
            
    def output(self, htmlDir):
        """Saves the match information and results as HTML page in the 
        directory 'htmlDir' and returns the filename."""
        if self.type == "Pairwise":
            baseName = self.type+"_"+self.strategies[0].func_name+\
                       "_"+self.strategies[1].func_name+"_"+str(self.matchNr)
        else:
            baseName = self.type+"_"+str(self.matchNr)
        htmlPage = Report.HTMLPage(htmlDir, baseName, baseName)
        htmlPage.heading(self.description, 1, False)
        self.ranking = dumpRanking(htmlPage, self.result)
        if CFG.REGISTER_FREQUENCIES:
            htmlPage.heading("Frequencies", 2, True)
            htmlPage.plot([s.func_name for s in self.strategies], 
                          self.frequencies, "rounds", "frequency", 
                          "Frequencies of the strategies")
        if CFG.REGISTER_PAYOFFS:
            htmlPage.heading("Payoffs", 2, True)
            htmlPage.plot([s.func_name for s in self.strategies],
                          self.payoffs, "rounds", "payoff", 
                          "Payoffs per individual")            
        if CFG.REGISTER_MEAN_PAYOFFS:
            htmlPage.heading("Mean Payoffs", 2, True)
            htmlPage.plot([s.func_name for s in self.strategies],
                          self.mean_payoffs, "rounds", "mean payoffs",
                          "Mean payoffs per individual")
        if CFG.POST_MORTEM_ANALYSIS:
            htmlPage.heading("Performance Profiles of the Strategies", 2,True)
            for freq, strategy in self.ranking:
                if self.forensics.has_key(strategy):
                    htmlPage.heading("Post Mortem Analysis of strategy %s"\
                                     %strategy.func_name, 3, True)
                    pms = self.forensics[strategy][0]
                    pms.output(htmlPage, 4)
                    for pmi in self.forensics[strategy][1:]:
                        pmi.output(htmlPage, 4)
        return htmlPage.saveToDisk()
        

def stripURL(url, htmlDir):
    """If 'url' starts with 'htmlDir' then remove 'htmlDir' from 'url' so
    that 'url' only contains the relative path."""
    if url.find(htmlDir) == 0: 
        url = url[len(htmlDir):]
        while (len(url) > 1 and (url[0] == "/" or url[0] == "\\")):
            url = url[1:]
        return url
    else: return url


class ContestData(object):
    """The 'MatchData' class collects the statistical information about
    one match (either pairwise or melee):
    
    strategies  - a list of the strategies
    type        - the type of the contest, either "Pairwise" or "Melee"
    result      - dictionary containing the average frequencies of the 
                  strategies during the last quarter of the matches
    resultA,
    resultB     - intermediate results for pairwise contests, that contain
                  the average results for these matches before and after
                  swapping the role of the invader 
    description - a string describing the contest, 
                  i.e. "Pairwise contest of strategies 'Good' and 'Bad'"
    forensics   - A dictionary with list of 'PostMortem' objects for each 
                  strategy
    matchURLs   - The list of the URLS of the html-pages describing the 
                  single matches of the contest
    ranking     - The final ranking of the contest
    rankings    - The list of rankings of each single match
                  
                  (some of the attributes are only present if the respective
                  configuration flags are set!)                  
    """
    def __init__(self, strategies, typeString):
        self.strategies = strategies
        self.type = typeString
        self.result = {}
        self.resultA = {}
        self.resultB = {}
        self.description = typeString + " contest nr. of strategies: " + \
                           "".join(["'"+s.func_name+"', " for s in strategies[:-1]]) +\
                           "'" + strategies[-1].func_name + "'"    
        if CFG.POST_MORTEM_ANALYSIS:
            self.forensics = {}
            for s in strategies: self.forensics[s] = []
        if CFG.MONITOR_MATCHES:
            self.matchURLs = []
            self.rankings = []
        self.ranking = []
        
    def addMatch(self, matchData, matchUrl):
        """Processes the data of a match of the contest after it has been 
        finished. 'matchData' is the MatchData object for the match and
        'matchUrl' is the URL of the html page that is returned by 
        matchData.output.
        """
        if CFG.POST_MORTEM_ANALYSIS:
            for s in self.strategies:
                try:
                    self.forensics[s].append(matchData.forensics[s][0])
                except KeyError: pass
        if CFG.MONITOR_MATCHES:
            self.matchURLs.append(matchUrl)
            self.rankings.append(matchData.ranking)
    
    def wrapItUp(self):
        """Finish processing data. If POST_MORTEM_ANALYSIS is set, the
        forensics data will be accumulated.
        """
        if CFG.POST_MORTEM_ANALYSIS:
            for strategy in self.forensics.keys():
                pmsList = self.forensics[strategy]
                if len(pmsList) >= 1:
                    self.forensics[strategy] = PostMortemSummary(pmsList)
                else: del self.forensics[strategy]
    
    def output(self, htmlDir):
        """Write a contest summary as HTML page. The generated HTML page
        references the html pages of all single matches.
        """
        def dumpMatchList(htmlPage, matchURLs, rankings):
            """Appends a list of links to the single matches to the html page.
            """
            for i,e in enumerate(zip(matchURLs, rankings)):
                url, ranking = e[0], e[1]
                text = "%i. Match (Winner: %s, %1.4f)" % \
                    (i+1, ranking[0][1].func_name, ranking[0][0]) 
                htmlPage.link(stripURL(url, htmlDir), text) 
                htmlPage.crlf()  
            htmlPage.crlf()        
        baseName = self.type+"_Contest"
        htmlPage = Report.HTMLPage(htmlDir, baseName, baseName)     
        htmlPage.heading(self.description, 1, False)
        self.ranking = dumpRanking(htmlPage, self.result)
        if self.type == "Pairwise": 
            table = [["<b>Defender</b>", "<b>Invader</b>"],
                     [self.strategies[0].func_name,
                      self.strategies[1].func_name],
                     ["%1.4f"%self.resultA[self.strategies[0]], 
                      "%1.4f"%self.resultA[self.strategies[1]]],
                     ["<b>Defender</b>", "<b>Invader</b>"],
                     [self.strategies[1].func_name,
                      self.strategies[0].func_name],
                     ["%1.4f"%self.resultB[self.strategies[1]], 
                      "%1.4f"%self.resultB[self.strategies[0]]]]
            htmlPage.table(array(table))
        if CFG.MONITOR_MATCHES:
            htmlPage.heading("Matches", 2, True)          
            if self.type == "Pairwise":
                m = CFG.PW_REPETITIONS/2
                htmlPage.heading("%s (Defender), %s (Invader)" % \
                                 (self.strategies[0].func_name,
                                  self.strategies[1].func_name), 3, True)
                dumpMatchList(htmlPage, self.matchURLs[:m], self.rankings[:m])                  
                htmlPage.heading("%s (Defender), %s (Invader)" % \
                                 (self.strategies[1].func_name,
                                  self.strategies[0].func_name), 3, True)                
                dumpMatchList(htmlPage, self.matchURLs[m:], self.rankings[m:])                              
            else:               
                dumpMatchList(htmlPage, self.matchURLs, self.rankings)                  
        if CFG.POST_MORTEM_ANALYSIS:
            htmlPage.heading("Performance Profiles of the Strategies", 2,True)
            for freq, strategy in self.ranking:
                if self.forensics.has_key(strategy):
                    htmlPage.heading("Post Mortem Analysis of strategy %s"\
                                     %strategy.func_name, 3, True)
                    pms = self.forensics[strategy]
                    pms.output(htmlPage, 4)
        return htmlPage.saveToDisk()



class TournamentData(object):
    def __init__(self, strategies):
        self.strategies = strategies
        self.description = "Tournament of strategies: " + \
                           "".join(["'"+s.func_name+"', " for s in strategies[:-1]]) +\
                           "'" + strategies[-1].func_name + "'" 
        self.contestURLs = []
        self.currContestStrategies = []
        self.contestResults = []
        self.meleeURL = ""
        self.meleeRanking = []
        self.phase1Ranking = []
        self.phase1Winners = []

    def newContest(self, strategies):
        self.currContestStrategies = strategies

    def addContest(self, url, result):
        self.contestURLs.append(url)
        s = self.currContestStrategies
        self.contestResults.append([(s[0].func_name,result[s[0]]),
                                    (s[1].func_name,result[s[1]])])
    
    def addMelee(self, url, result):
        self.meleeURL = url
        self.meleeRanking = result
            
    def phase1Result(self, strategies, ranking):
        self.phase1Ranking = ranking
        self.phase1Winners = strategies

    def wrapItUp(self):
        pass
    
    def output(self, htmlDir):
        htmlPage = Report.HTMLPage(htmlDir, "Tournament", "Tournament")
        htmlPage.heading(self.description, 1, False)        
        dumpRanking(htmlPage, self.meleeRanking)
        htmlPage.link(stripURL(self.meleeURL, htmlDir), \
                      "Final Round: Detailed Results")
        htmlPage.crlf()
        htmlPage.crlf()
        
        htmlPage.heading("First Round", 2)
        htmlPage.heading("Ranking", 3)
        htmlPage.text('<div style="font-family: monospace;">')
        htmlPage.openNumberedList()
        i = 1
        for won, strategy in self.phase1Ranking:
            s = list(strategy.func_name)
            s = "".join((s + ["&nbsp;"]*(30-len(s)-int(math.log10(i))))[0:40])
            if i <= 10:
                htmlPage.listItem("<b>%s  %3i</b>"% (s, won))
            else:
                htmlPage.listItem("%s  %3i"% (s, won))
            i += 1
        htmlPage.closeNumberedList()
        htmlPage.text('</div>')        
        
        htmlPage.heading("First Round: Detailed Results", 3)
        htmlPage.text('<div style="font-family: monospace;">')
        i = 1
        for r1, r2 in self.contestResults:
            s = list(r1[0]+ " : "+r2[0])
            s = "".join((s + ["&nbsp;"]*(60-len(s)))[0:60])
            htmlPage.link(stripURL(self.contestURLs[i-1], htmlDir), 
                          "%3i. %s %1.2f:%1.2f" % (i, s, r1[1], r2[1]))
            htmlPage.crlf()
            i += 1
        htmlPage.text('</div>')
        htmlPage.crlf()
        return htmlPage.saveToDisk()      
        
        

def CreateDirSet(baseDir, dirName):
    """Creates directory 'dirName' and subdirectory 'dirName/images' in 
    directory 'baseDir'"""
    global output_path
    path = os.path.join(baseDir, dirName)
    while os.path.exists(path):
        dirName += "_"  # poor hack!
        path = os.path.join(baseDir, dirName)
    os.mkdir(path)
    os.mkdir(os.path.join(path, "images"))
    output_path = path
    return path
                    

def CreateGenericDir(baseName = "cultaptation"):
    genericName = baseName+"_".join([str(i) for i in (time.localtime()[:5])])
    mainDir = os.path.join(os.path.expanduser("~"), "cultaptation_output")
    if not os.path.exists(mainDir): os.mkdir(mainDir)
    path = os.path.join(mainDir, genericName)
    return CreateDirSet(path, "")


def PrepareTournamentDirSet(baseName = "cultaptation"):
    global output_path, tournament_path, phase1_path, phase2_path
    tournament_path = CreateGenericDir()
    phase1_path = CreateDirSet(tournament_path, "Contests")
    phase2_path = CreateDirSet(tournament_path, "Melee")
    return tournament_path

def TournamentNewContestDir(contestName):
    global phase1_path
    return CreateDirSet(phase1_path, contestName)
    
def TournamentMeleeDir():
    global output_path, phase2_path
    output_path = phase2_path
    return output_path

#
# Match recording:
#

def PWStart(initialEnvrionment, strategies, population):
    global data, match_counter, round_counter
    match_counter += 1
    round_counter = 0
    resetLiveGraph()
    if CFG.MONITOR_MATCHES or CFG.POST_MORTEM_ANALYSIS:
        data = MatchData(strategies, "Pairwise", match_counter)

def MMStart(initialEnvrionment, strategies, population):
    global data, match_counter, round_counter
    match_counter += 1
    round_counter = 0
    resetLiveGraph()
    if CFG.MONITOR_MATCHES or CFG.POST_MORTEM_ANALYSIS:
        data = MatchData(strategies, "Melee", match_counter)    


def Record(population):
    global data, base_counter, match_counter, round_counter
    round_counter += 1
    if not CFG.MONITOR_MATCHES:
        if CFG.POST_MORTEM_ANALYSIS: data.round += 1
    else:  data.record(population)
    progressCallback(match_counter+base_counter, round_counter)
    
PWRecord = Record
MMRecord = Record

#
# Contest Recording
#

def PCStart(strategyA, strategyB):
    global match_counter, matchCollection, contest_data
    match_counter = 0
    matchCollection = []
    contest_data = ContestData([strategyA, strategyB], "Pairwise")

def PCSwap(frequencies):
    if not contest_data.resultA:
        contest_data.resultA = frequencies
    else:
        contest_data.resultB = frequencies 

def ContestRecord(frequencies):
    global data
    if CFG.MONITOR_MATCHES or CFG.POST_MORTEM_ANALYSIS:
        data.result = frequencies
        data.wrapItUp()
        url = data.output(output_path)
        if contest_data: contest_data.addMatch(data, url)
    else: url = ""
    return url

PCRecord = ContestRecord
MCRecord = ContestRecord

def MCStart(strategies):
    global match_counter, matchCollection, contest_data
    match_counter = 0
    matchCollection = []
    contest_data = ContestData(strategies, "Melee")

def ContestFinished(frequencies):
    global contest_data
    contest_data.result = frequencies
    contest_data.wrapItUp()
    return contest_data.output(output_path)

PCFinished = ContestFinished
MCFinished = ContestFinished


#
# Tournament Recording
#


def TournamentStart(strategies):
    global tournament_data, base_counter
    base_counter = 0
    tournament_data = TournamentData(strategies)

def TournamentPhase1(strategyA, strategyB):
    global tournament_data
    strategies = [strategyA, strategyB]
    tournament_data.newContest(strategies)    
    TournamentNewContestDir(strategies[0].func_name + "-" + strategies[1].func_name)
    
def TournamentPhase1Record(frequencies):
    global tournament_data, base_counter
    base_counter += 1    
    tournament_data.addContest(PCFinished(frequencies), frequencies)

def TournamentPhase2(strategies, ranking):
    global tournament_data    
    tournament_data.phase1Result(strategies, ranking)
    TournamentMeleeDir()

def TournamentFinished(frequencies):
    global tournament_data, base_counter
    base_counter = 0    
    tournament_data.addMelee(MCFinished(frequencies), frequencies)
    tournament_data.wrapItUp()
    return tournament_data.output(tournament_path)
    