"""Copyright (c) 2008 by Eckhart Arnold (www.eckhartarnold.de)
                   
License: GNU Public License V3
(http://www.gnu.de/documents/gpl-3.0.en.html)

This program is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the 
Free Software Foundation, either version 3 of the License, or (at your 
option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program. If not, see <http://www.gnu.org/licenses/>.
"""


def DynamicAspirationLevel(roundsAlive, myRepertoire, myHistory):
    """by Eckhart Arnold
    
    DynamicAspirationLevel determines an aspiration level that is
    slightly higher than the mean payoff of all previous EXPLOIT
    moves. The difference between the maximun payoff in the 
    repertoire and the aspiration level determines the probability 
    with which an OBSERVE move is chosen vs. an EXPLOIT move.
    
    Furthermore, "DynamicAspirationLevel" uses a 'decaying repertoire' 
    where old entries and potentially invalid entries are values less 
    than new ones.
    
    The strategy in detail:
    
    The strategy "DynamicAspirationLevel" consists of two phases, the 
    first of which lasts until the fourth round of the life time of 
    an individual using this strategy. The second phase last during 
    the remainder of the life time of the individual. 
    
    1. Phase One:
    
    - Play OBSERVE in the first round alive (when 'roundsAlive = 0')
    - In the second round alive: If the repertoire is still empty (i.e. nothing
      could be oserved in the first round) play INNOVATE, otherwise chose
      the best EXPLOIT move from the repertoire.
    - In the third round alive, play OBSERVE
    - In the fourth round, chose the best move from the repertoire
    
    2. Phase Two

    - determine the mean payoff 'm' from all EXPLOIT moves in the history.
    - determine the maximum payoff 'M' from all EXPLOIT moves in the history.
    - calculate an aspiration level 'a' according to the magic formula:
                            a := m + (M-m) / 5
    - determine the highest payoff 'ace' in the repertoire.
    - if the highest payoff in the repertoire is greater or equal the
      aspiration level ('ace >= a') then set the observataion probability 'p'
      to 0.01. 
      Otherwise, if the aspiration level 'a' is greater zero,
      let the observation probability 'p' be the maximum 
      of 0.01 and the difference between the aspiration level and the 
      highest payoff in the repertoire, divided by the aspiration level '(a-ace)/a'.
      Otherwise, if the aspiration level is zero, let 'p' be one.
       
    - generate a random number 'r' between 0 and 1. If the random number is less
      or equal the observation probability 'p' chose an OBSERVATION move with a 
      chance of 99% and an INNOVATE move with a chance of 1%.
      Always choose INNOVATE if the aspiration level 'a' was not greater zero!
      Otherwise chose the best EXPLOIT move from the repertoire.
      
    - Decay the repertoire by subtracting 1 from each payoff value registered
      in the repertoire.

    """
    
    # phase 1
    
    if roundsAlive == 0:        
        move = OBSERVE
        
    elif roundsAlive == 1: 
        if myRepertoire.shape[1] == 0: 
            move = INNOVATE
        else:
            move = bestMove(myRepertoire)
            
    elif roundsAlive == 2:
        move = OBSERVE
        
    elif roundsAlive == 3:
        move = bestMove(myRepertoire)
      
    # phase 2
        
    else:
        meanP = meanPayoff(myHistory)
        maxP = maxPayoff(myHistory)
        aspirationLevel = meanP + (maxP-meanP) / 5.0 
        ace = repMaxPayoff(myRepertoire)
              
        if ace >= aspirationLevel: 
            p = 0.01              
        elif aspirationLevel > 0.0:
            p = max(0.01, (aspirationLevel-ace) / aspirationLevel)
        else: p = 1.0
            
        if rnd() <= p:
            if aspirationLevel <= 0.0 or rnd() <= 0.01:  
                move = INNOVATE
            else:
                move = OBSERVE
        else:
            move = bestMove(myRepertoire)
        
    decayRepertoire(myRepertoire)
    return (move, myRepertoire)



def StaticAspirationLevel(roundsAlive, myRepertoire, myHistory):
    """by Eckhart Arnold
    
    The strategy "StaticAspirationLevel" uses the mean payoff value from the
    first ten rounds as an aspiration level. The difference between the best
    payoff in the repertoire and the aspiration level determines the probability
    with which an OBSERVE move is played.

    Furthermore, "StaticAspirationLevel" employs a 'decaying repertoire' 
    where old entries and potentially invalid entries are values less 
    than new ones.
    
    The strategy in detail:
    
    The strategy "StaticAspirationLevel" consists of two phases, the 
    first of which lasts until the fourth round of the life time of 
    an individual using this strategy. The second phase last during 
    the remainder of the life time of the individual. 
    
    1. Phase One:
    
    - Play OBSERVE in the first round (i.e. when 'roundsAlive = 0').
    - If the repertoire is still empty in the second round, play INNOVATE.
    - In the third and fourth round, play OBSERVE if the repertoire contains
      fewer than five entries and if the last move was neither OBSERVE nor
      INNOVATE. Play the best move from the repertoire otherwise.
    - occasionally (with a probability of 1%) turn an OBSERVE move into an
      INNOVATE move
    
    2. Phase Two:
    
    - set the aspiration level 'a' to the mean payoff gained from all EXPLOIT
      moves during the first 10 rounds or during all previous rounds if the life
      time is less than 10 rounds.
    - if the highest payoff in the repertoire ('ace') is better than the aspiration
      level (i.e. if 'ace >= a') then chose the best EXPLOIT move in the repertoire
    - otherwise, if the aspiration level is greater zero,
      determine an observation probability 'p' according to the formula:
      '(a-ace) / a', i.e. the difference of the aspiration level and the best payoff
      divided by the aspiration level.
    - if the aspiration level 'a' was zero then choose INNOVATE as move and skip 
      to the last step
    - pick a random value 'r' between zero and one. If the random value is smaller
      or equal the observation probability ('r <= p') chose OBSERVE. Chose the
      best EXPLOIT move according to the repertoire otherwise. 
    - occasionally (with a probability of 1%) turn an OBSERVE move into an
      INNOVATE move
    - Decay the repertoire by subtracting 1 from each payoff value registered
      in the repertoire.    
    """
    if roundsAlive == 0:        
        move = OBSERVE
        
    elif myRepertoire.shape[1] == 0: # should almost never happen except in the beginning of a pairwise match
        move = INNOVATE
        
    elif roundsAlive < 4:
        if myRepertoire.shape[1] < 5 and lastAct(myHistory) not in [OBSERVE, INNOVATE]:
            move = OBSERVE
        else: 
            move = bestMove(myRepertoire)
        
    else:
        rep = myRepertoire # genRepertoire(tail(myHistory, 15)) # ignore stale repertoire entries
        aspiration = meanPayoff(head(myHistory,10)) # aspiration level is the mean of the
        ace = repMaxPayoff(rep)      
        if ace >= aspiration:
            move = bestMove(rep)
        elif aspiration > 0.0:
            p = (aspiration-ace) / float(aspiration)
            if rnd() <= p:
                move = OBSERVE
            else:
                move = bestMove(rep)
        else:
            move = INNOVATE
                
    if move == OBSERVE and rnd() < 0.01: # occasionally turn an OBSERVE move into an INNOVATE move
        move = INNOVATE
    decayRepertoire(myRepertoire)
    return (move, myRepertoire)
  



def Childhood(roundsAlive, myRepertoire, myHistory):
    """idea by Martin Neuman, implemented by Eckhart Arnold

    The strategy Childhood observes more often during the early rounds of 
    its life time to fill the repertoire and seldom afterwards.
    
    The strategy in detail:
    
    - Play OBSERVE in the first, third and seventh round.
    - Play OBSERVE every 15th round.
    - Turn an OBSERVE move into an INNOVATE move with a probability of 1%    
    - Play INNOVATE if the repertoire is still empty after the first round.
    - In all othter cases chose the best EXPLOIT move from the repertoire.
    """
    if roundsAlive in [0, 2, 6]:
        move = OBSERVE
    elif myRepertoire.shape[1] == 0:
        move = INNOVATE
    elif roundsAlive % 15 == 0 or repMaxPayoff(myRepertoire) == 0:
        move = OBSERVE
    else:
        move = bestMove(myRepertoire)
    if move == OBSERVE and rnd() < 0.01: move = INNOVATE
    return (move, myRepertoire)   