# -*- coding: utf-8 -*-

"""cultaptation Strategies

Copyright (c) 2008 by Eckhart Arnold (www.eckhartarnold.de)
                   portions by Julia Lohmann, Philipp Schächtele
                   Christoph Siemroth, Martin Neumann
                   
License: GNU Public License V3
(http://www.gnu.de/documents/gpl-3.0.en.html)

This program is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the 
Free Software Foundation, either version 3 of the License, or (at your 
option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from random import random as rnd
from numpy import argsort, where, mean
from Configuration import *
from Toolkit import *




def ExampleStrategy(roundsAlive, myRepertoire, myHistory):
    """The Example strategy from the 'rules for entry' document. 
    Starts with INNOVATE in the first round then plays EXPLOIT
    (chhosing the best move in the repertoire) until the last
    move is smaller than the average move. In this case OBSERVE
    is played. OBSERVE is never played to times in sequence but
    always followed by an exploit move."""
    if roundsAlive > 1:
        myMeanPayoff = meanPayoff(myHistory)
        lastPayoff = payoffs(exploits(myHistory))[-1]
        if lastPayoff < myMeanPayoff and lastAct(myHistory) != OBSERVE:
            move = OBSERVE
        else:
            move = bestMove(myRepertoire)
    elif roundsAlive > 0:
        move = myRepertoire[0, 0]
    else:
        move = INNOVATE
    return (move, myRepertoire)


###############################################################################
#
# Strategies by Philipp Schächtele
#
###############################################################################

def Inventor(roundsAlive, myRepertoire, myHistory):
    """see Philipp Schaechtele's netlogo code"""
    if roundsAlive < 1:
        move = INNOVATE
    else:
        if roundsAlive % 10 == 0:
            move = INNOVATE
        else:
            move = bestMove(myRepertoire)
    return (move, myRepertoire)

    
def Observer(roundsAlive, myRepertoire, myHistory):
    """see Philipp Schaechtele's netlogo code"""
    if roundsAlive < 1:
        move = INNOVATE
    else:
        if rnd() < 1.0/7.0:
            move = OBSERVE
        else:
            move = bestMove(myRepertoire)
    return (move, myRepertoire)


###############################################################################
#
# Strategies by Christoph Siemroth
#
###############################################################################

def SPYnWORK7(roundsAlive, myRepertoire, myHistory):
    """CS"""
    m,n = myRepertoire.shape
    if roundsAlive < 2: 
        move = OBSERVE
    elif n == 0:
        move = INNOVATE
    elif roundsAlive == 2:
        move = bestMove(myRepertoire)
    else:
        rep = genRepertoire(tail(myHistory, 25))
        performance = sum(payoffs(exploits(myHistory))) / (roundsAlive-2) # warum -2 
        ace = repMaxPayoff(rep)
        if ace < performance:
            move = OBSERVE
        else:
            move = bestMove(rep)
    return (move, myRepertoire)

def SPYnWORK8(roundsAlive, myRepertoire, myHistory):
    """CS"""
    m,n = myRepertoire.shape
    if roundsAlive < 2: 
        move = OBSERVE
    elif n == 0:
        move = INNOVATE
    elif roundsAlive == 2:
        move = bestMove(myRepertoire)
    else:
        rep = genRepertoire(tail(myHistory, 25))
        performance = sum(payoffs(exploits(myHistory))) / (roundsAlive-2) # warum -2 
        ace = repMaxPayoff(rep)
        lazy = 0
        for act in acts(fold(myHistory))[-1::-1]:
            if act >= EXPLOIT_MIN: break
            lazy += 1
        if lazy > 2:
            move = bestMove(rep)
        else:
            if ace < performance:
                move = OBSERVE
            else:
                move = bestMove(rep)
    return (move, myRepertoire)

def Worker(roundsAlive, myRepertoire, myHistory):
    """by Christoph Siemroth
    
    + exploits alot and should have a high average payoff especially when 
      environment changes often (that is, pc is high), also, invidivuals dying
      soon contribute to the average payoff appropriately
    - might have low payoff in second stage when observing of several 
      invididuals is allowed
    """
    if roundsAlive < 4: 
        move = INNOVATE
    else:
        move = bestMove(myRepertoire)
    return (move, myRepertoire)


def SmartWorker(roundsAlive, myRepertoire, myHistory):
    """by Christoph Siemroth
    
    + exploits alot and shifts (=innovates) only if the highest available 
      payoff is too low compared to its experience so far, thereby keeping the
      repertoire efficient from its subjective perspective
    + doesn’t waste too much rounds by innovating but is forced to use the 
      results in due time
    - might have low payoff in second stage when observing of several 
      invididuals is allowed    
    """
    if roundsAlive < 4: 
        move = INNOVATE
    elif roundsAlive == 4:
        move = bestMove(myRepertoire)
    else:
        performance = sum(payoffs(exploits(myHistory))) / (roundsAlive-2) # warum -2 
        ace = repMaxPayoff(myRepertoire)
        lazy = 0
        for act in acts(fold(myHistory))[-1::-1]:
            if act >= EXPLOIT_MIN: break
            lazy += 1
        if lazy > 2:
            move = bestMove(myRepertoire)
        else:
            if ace < performance:
                move = INNOVATE
            else:
                move = bestMove(myRepertoire)
    return (move, myRepertoire)


###############################################################################
#
# Strategies by Julia Lohmann
#
###############################################################################

def Observer_julia(roundsAlive, myRepertoire, myHistory):
    """spielt überhaupt kein INNOVATE, beobachtet in Runde 1 - 10
       und spielt dann immer EXPLOIT mit dem act mit dem höchsten payoff;
       reine OBSERVER setzten sich nicht durch!!!!"""
    if roundsAlive < 11:
        move = OBSERVE
    elif myRepertoire.shape[1] == 0: # should almost never happen
        move = INNOVATE        
    else:
        move = bestMove(myRepertoire)
    return (move, myRepertoire) 


def InnovateObserver_julia(roundsAlive, myRepertoire, myHistory):
    """spielt in den ersten 5 Runden INNOVATE, beobachtet in Runde 6 - 10
       und spielt dann immer EXPLOIT mit dem act mit dem höchsten payoff"""
    if roundsAlive < 6:
        move = INNOVATE
    else:
        if roundsAlive < 11:
            move = OBSERVE
        else:
            move = bestMove(myRepertoire)
    return (move, myRepertoire)


def MixedBag_julia(roundsAlive, myRepertoire, myHistory):
    """PLAYS alternating INNOVATE and OBSERVE moves in every second
    round during the first 20 rounds with EXPLOIT moves in between. 
    After the 20th EXPLOIT is always played."""
    if roundsAlive < 20:
        if roundsAlive in [0, 4, 8, 12, 16]:
            move = INNOVATE
        elif roundsAlive in [2, 6, 10, 14, 18]:
            move = OBSERVE
        else:
            move = bestMove(myRepertoire)
    else:
        move = bestMove(myRepertoire)
    return (move, myRepertoire)
        
    

###############################################################################
#
# Strategies by Eckhart Arnold
#
###############################################################################



def RefinedAverageChecker(roundsAlive, myRepertoire, myHistory):
    """During the first six rounds OBSERVE, if the repertoire contains 
    less than 6 entries and if the last move wasn't OBSERVE already.
    After that, if the last payoff was smaller than the the average
    payoff, and the previous move was not an OBSERVE move, pick the best
    move from the repertoire if there still is a move stored that is better 
    than the last move. Otherwise OBSERVE.
    For the choice of the exploit move, do only consider payoff information
    that ist not to old ("stale"). 
    """
    if roundsAlive == 0:
        move = OBSERVE
    elif myRepertoire.shape[1] == 0: # should almost never happen
        move = INNOVATE
    elif roundsAlive < 6:
        if myRepertoire.shape[1] < 6 and lastAct(myHistory) not in [OBSERVE, INNOVATE]:
            move = OBSERVE
        else:
            move = bestMove(myRepertoire)
    else:
        rep = myRepertoire # genRepertoire(tail(myHistory, 15)) # ignore stale repertoire entries      
        if lastAct(myHistory) not in [OBSERVE, INNOVATE]:
            reference = meanPayoff(myHistory)
            ace = repMaxPayoff(rep)
            lastPayoff = myHistory[3, -1]          
            if lastPayoff < reference and ace <= lastPayoff:
                move = OBSERVE
            else:
                move = bestMove(rep)
        else: 
            move = bestMove(rep)
    if move == OBSERVE and rnd() < 0.01:
        move = INNOVATE                    # occasionally turn an OBSERVE move into an INNOVATE move!
    decayRepertoire(myRepertoire)
    return (move, myRepertoire)



def AspirationLevel(roundsAlive, myRepertoire, myHistory):
    """During the first four rounds OBSERVE, if the repertoire contains 
    less than 6 entries and if the last move wasn't OBSERVE already.
    In the following rounds use the mean of the EXPLOIT moves during the first
    10 rounds as an aspiration level. OBSERVE whenever the last payoff falls
    below the aspiration level, but do not OBSERVE two times in sequence.
    Occasionally (1%) turn an intended OBSERVE move into an INNOVATE move.
    """
    if roundsAlive == 0:        
        move = OBSERVE
        
    elif myRepertoire.shape[1] == 0: # should almost never happen except in the beginning of a pairwise match
        move = INNOVATE
        
    elif roundsAlive < 5:
        if myRepertoire.shape[1] < 6 and lastAct(myHistory) not in [OBSERVE, INNOVATE]:
            move = OBSERVE
        else: 
            move = bestMove(myRepertoire)
        
    else:
        rep = myRepertoire # genRepertoire(tail(myHistory, 15)) # ignore stale repertoire entries
        if lastAct(myHistory) not in [OBSERVE, INNOVATE]:   # make sure that there are two exploit moves after every OBSERVE move
            aspiration = meanPayoff(head(myHistory,10)) # aspiration level is the mean of the first 10 moves
            ace = repMaxPayoff(rep)      
            if ace >= aspiration:
                move = bestMove(rep)
            else:
                move = OBSERVE
        else:
            move = bestMove(rep)
    if move == OBSERVE and rnd() < 0.01: # occasionally turn an OBSERVE move into an INNOVATE move
        move = INNOVATE
    return (move, myRepertoire)



#def DynamicAspirationLevelB(roundsAlive, myRepertoire, myHistory):
#    """During the first six rounds OBSERVE, if the repertoire contains 
#    less than 6 entries and if the last move wasn't OBSERVE already.
#    In the following rounds use the mean of the EXPLOIT moves during the first
#    10 rounds as an aspiration level. OBSERVE when the highest payoff in the
#    repertoire of the last 10 rounds (the "ace") is smaller than the aspiration level and
#    a random number is greater than (("aspiration level" minus "ace") / "aspiration level")).
#    Occasionally (0.5%) turn an intended OBSERVE move into an INNOVATE move.
#    """
#    
#    # phase 1
#    
#    if roundsAlive == 0:        
#        move = OBSERVE
#        
#    elif myRepertoire.shape[1] == 0: # should almost never happen except in the beginning of a pairwise match
#        move = INNOVATE
#        
#    elif roundsAlive < 4:
#        if myRepertoire.shape[1] < 5 and lastAct(myHistory) not in [OBSERVE, INNOVATE]:
#            move = OBSERVE
#        else: 
#            move = bestMove(myRepertoire)
#     
#    # phase 2
#        
#    else:
#        meanP = meanPayoff(myHistory)
#        maxP = maxPayoff(myHistory)
#        aspirationLevel = meanP + (maxP-meanP) / 5 # aim a little higher than the mean!
#        ace = max(0, repMaxPayoff(myRepertoire))
#              
#        if ace >= aspirationLevel: 
#            p = 0.001               # occasionally OBSERVE at random
#        else: 
#            p = max(0.001, (aspirationLevel-ace) / float(aspirationLevel))
#            
#        if rnd() <= p:
#            if rnd() <= 0.01:   # occasionally turn an OBSERVE into an INNOVATE move
#                move = INNOVATE
#            else:
#                move = OBSERVE
#        else:
#            move = bestMove(myRepertoire)
#        
#    decayRepertoire(myRepertoire)
#    return (move, myRepertoire)


def DynamicAspirationLevel(roundsAlive, myRepertoire, myHistory):
    """by Eckhart Arnold
    
    DynamicAspirationLevel determines an aspiration level that is
    slightly higher than the mean payoff of all previous EXPLOIT
    moves. The difference between the maximun payoff in the 
    repertoire and the aspiration level determines the probability 
    with which an OBSERVE move is chosen vs. an EXPLOIT move.
    
    Furthermore, "DynamicAspirationLevel" uses a 'decaying repertoire' 
    where old entries and potentially invalid entries are values less 
    than new ones.
    
    The strategy in detail:
    
    The strategy "DynamicAspirationLevel" consists of two phases, the 
    first of which lasts until the fourth round of the life time of 
    an individual using this strategy. The second phase last during 
    the remainder of the life time of the individual. 
    
    1. Phase One:
    
    - Play OBSERVE in the first round alive (when 'roundsAlive = 0')
    - In the second round alive: If the repertoire is still empty (i.e. nothing
      could be oserved in the first round) play INNOVATE, otherwise chose
      the best EXPLOIT move from the repertoire.
    - In the third round alive, play OBSERVE
    - In the fourth round, chose the best move from the repertoire
    
    2. Phase Two

    - determine the mean payoff 'm' from all EXPLOIT moves in the history.
    - determine the maximum payoff 'M' from all EXPLOIT moves in the history.
    - calculate an aspiration level 'a' according to the magic formula:
                            a := m + (M-m) / 5
    - determine the highest payoff 'ace' in the repertoire.
    - if the highest payoff in the repertoire is greater or equal the
      aspiration level ('ace >= a') then set the observataion probability 'p'
      to 0.01. 
      Otherwise, if the aspiration level 'a' is greater zero,
      let the observation probability 'p' be the maximum 
      of 0.01 and the difference between the aspiration level and the 
      highest payoff in the repertoire, divided by the aspiration level '(a-ace)/a'.
      Otherwise, if the aspiration level is zero, let 'p' be one.
       
    - generate a random number 'r' between 0 and 1. If the random number is less
      or equal the observation probability 'p' chose an OBSERVATION move with a 
      chance of 99% and an INNOVATE move with a chance of 1%.
      Always choose INNOVATE if the aspiration level 'a' was not greater zero!
      Otherwise chose the best EXPLOIT move from the repertoire.
      
    - Decay the repertoire by subtracting 1 from each payoff value registered
      in the repertoire.

    """
    
    # phase 1
    
    if roundsAlive == 0:        
        move = OBSERVE
        
    elif roundsAlive == 1: 
        if myRepertoire.shape[1] == 0: 
            move = INNOVATE
        else:
            move = bestMove(myRepertoire)
            
    elif roundsAlive == 2:
        move = OBSERVE
        
    elif roundsAlive == 3:
        move = bestMove(myRepertoire)
      
    # phase 2
        
    else:
        meanP = meanPayoff(myHistory)
        maxP = maxPayoff(myHistory)
        aspirationLevel = meanP + (maxP-meanP) / 5.0 
        ace = repMaxPayoff(myRepertoire)
              
        if ace >= aspirationLevel: 
            p = 0.01              
        elif aspirationLevel > 0.0:
            p = max(0.01, (aspirationLevel-ace) / aspirationLevel)
        else: p = 1.0
            
        if rnd() <= p:
            if aspirationLevel <= 0.0 or rnd() <= 0.01:  
                move = INNOVATE
            else:
                move = OBSERVE
        else:
            move = bestMove(myRepertoire)
        
    decayRepertoire(myRepertoire)
    return (move, myRepertoire)



def DynamicObservationAndInnovation(roundsAlive, myRepertoire, myHistory):
    """by Eckhart Arnold
    
    DynamicObservationAndInnovation determines an aspiration level that is
    slightly higher than the mean payoff of all previous EXPLOIT
    moves. The difference between the maximun payoff in the 
    repertoire and the aspiration level determines the probability 
    with which an OBSERVE move is chosen vs. an EXPLOIT move.
    At the same time, the higher this probability the higher is
    also the probability with which an INNOVATE move is played 
    instead of an observe move.
    
    Furthermore, "DynamicAspirationLevel" uses a 'decaying repertoire' 
    where old entries and potentially invalid entries are values less 
    than new ones.
    
    The strategy in detail:
    
    The strategy "DynamicAspirationLevel" consists of two phases, the 
    first of which lasts until the fourth round of the life time of 
    an individual using this strategy. The second phase last during 
    the remainder of the life time of the individual. 
    
    1. Phase One:
    
    - Play OBSERVE in the first round alive (when 'roundsAlive = 0')
    - In the second round alive: If the repertoire is still empty (i.e. nothing
      could be oserved in the first round) play INNOVATE, otherwise chose
      the best EXPLOIT move from the repertoire.
    - In the third round alive, play OBSERVE
    - In the fourth round, chose the best move from the repertoire
    
    2. Phase Two
    
    General Remark: There exists and "observation" and an 
    "innovation probability". The "observation probability" is the probability
    with which either OBSERVE or(!) INNOVATE is played. If OBSERVE or INNOVATE 
    is played, the innovation probability determines whether it is really
    INNOVATE that is played. 

    - determine the mean payoff 'm' from all EXPLOIT moves in the history.
    - determine the maximum payoff 'M' from all EXPLOIT moves in the history.
    - calculate an aspiration level 'a' according to the magic formula:
                            a := m + (M-m) / 5
    - determine the highest payoff 'ace' in the repertoire.
    - if the highest payoff in the repertoire is greater or equal the the
      aspiration level ('ace => a') and greater or equal one ('ace >= 1') 
      then set the observation probability 'p' to 0.01 and the innovation 
      probability to 0.0001.
      
      Otherwise, if the aspiration level 'a' is greater or equal one,
      let the observation probability 'p' be the maximum 
      of 0.01 and the difference between the aspiration level and the 
      highest payoff in the repertoire, divided by the aspiration level 
      '(a-ace)/a'. Let the innovation probability 'p_inno' be
      the squared observation probability 'p_inno = p*p'.
      
      Otherwise, if the aspiration level is zero, let 'p' be one and
      let 'p_inno' be one minus the aspiration level (which at this point of
      the program is sure to be is smaller than one and greater than zero)
       
    - generate a random number 'r' between 0 and 1. If the random number is less
      or equal the observation probability 'p' chose an INNOVATION move with
      the innovation probability and an observation move otherwise.
      Always choose INNOVATE if the aspiration level 'a' was not greater zero!
      Otherwise chose the best EXPLOIT move from the repertoire.
      
    - Decay the repertoire by subtracting 1 from each payoff value registered
      in the repertoire.

    """
    
    # phase 1
    
    if roundsAlive == 0:        
        move = OBSERVE
        
    elif roundsAlive == 1: 
        if myRepertoire.shape[1] == 0: 
            move = INNOVATE
        else:
            move = bestMove(myRepertoire)
            
    elif roundsAlive == 2:
        move = OBSERVE
        
    elif roundsAlive == 3:
        move = bestMove(myRepertoire)
      
    # phase 2
        
    else:
        meanP = meanPayoff(myHistory)
        maxP = maxPayoff(myHistory)
        aspirationLevel = meanP + (maxP-meanP) / 5.0 
        ace = repMaxPayoff(myRepertoire)
              
        if ace >= 1 and ace >= aspirationLevel: 
            p = 0.01
            p_inno = 0.0001             
        elif aspirationLevel >= 1.0:
            p = max(0.01, (aspirationLevel-ace) / aspirationLevel)
            p_inno = p*p
        else: 
            p = 1.0
            p_inno = 1.0 - aspirationLevel
            
        if rnd() <= p:
            if rnd() <= p_inno:  
                move = INNOVATE
            else:
                move = OBSERVE
        else:
            move = bestMove(myRepertoire)
        
    decayRepertoire(myRepertoire)
    return (move, myRepertoire)



def StaticAspirationLevel(roundsAlive, myRepertoire, myHistory):
    """by Eckhart Arnold
    
    The strategy "StaticAspirationLevel" uses the mean payoff value from the
    first ten rounds as an aspiration level. The difference between the best
    payoff in the repertoire and the aspiration level determines the probability
    with which an OBSERVE move is played.

    Furthermore, "StaticAspirationLevel" employs a 'decaying repertoire' 
    where old entries and potentially invalid entries are values less 
    than new ones.
    
    The strategy in detail:
    
    The strategy "StaticAspirationLevel" consists of two phases, the 
    first of which lasts until the fourth round of the life time of 
    an individual using this strategy. The second phase last during 
    the remainder of the life time of the individual. 
    
    1. Phase One:
    
    - Play OBSERVE in the first round (i.e. when 'roundsAlive = 0').
    - If the repertoire is still empty in the second round, play INNOVATE.
    - In the third and fourth round, play OBSERVE if the repertoire contains
      fewer than five entries and if the last move was neither OBSERVE nor
      INNOVATE. Play the best move from the repertoire otherwise.
    - occasionally (with a probability of 1%) turn an OBSERVE move into an
      INNOVATE move
    
    2. Phase Two:
    
    - set the aspiration level 'a' to the mean payoff gained from all EXPLOIT
      moves during the first 10 rounds or during all previous rounds if the life
      time is less than 10 rounds.
    - if the highest payoff in the repertoire ('ace') is better than the aspiration
      level (i.e. if 'ace >= a') then chose the best EXPLOIT move in the repertoire
    - otherwise, if the aspiration level is greater zero,
      determine an observation probability 'p' according to the formula:
      '(a-ace) / a', i.e. the difference of the aspiration level and the best payoff
      divided by the aspiration level.
    - if the aspiration level 'a' was zero then choose INNOVATE as move and skip 
      to the last step
    - pick a random value 'r' between zero and one. If the random value is smaller
      or equal the observation probability ('r <= p') chose OBSERVE. Chose the
      best EXPLOIT move according to the repertoire otherwise. 
    - occasionally (with a probability of 1%) turn an OBSERVE move into an
      INNOVATE move
    - Decay the repertoire by subtracting 1 from each payoff value registered
      in the repertoire.    
    """
    if roundsAlive == 0:        
        move = OBSERVE
        
    elif myRepertoire.shape[1] == 0: # should almost never happen except in the beginning of a pairwise match
        move = INNOVATE
        
    elif roundsAlive < 4:
        if myRepertoire.shape[1] < 5 and lastAct(myHistory) not in [OBSERVE, INNOVATE]:
            move = OBSERVE
        else: 
            move = bestMove(myRepertoire)
        
    else:
        rep = myRepertoire # genRepertoire(tail(myHistory, 15)) # ignore stale repertoire entries
        aspiration = meanPayoff(head(myHistory,10)) # aspiration level is the mean of the
        ace = repMaxPayoff(rep)      
        if ace >= aspiration:
            move = bestMove(rep)
        elif aspiration > 0.0:
            p = (aspiration-ace) / float(aspiration)
            if rnd() <= p:
                move = OBSERVE
            else:
                move = bestMove(rep)
        else:
            move = INNOVATE
                
    if move == OBSERVE and rnd() < 0.01: # occasionally turn an OBSERVE move into an INNOVATE move
        move = INNOVATE
    decayRepertoire(myRepertoire)
    return (move, myRepertoire)
  

def Estimate(roundsAlive, myRepertoire, myHistory):
    """If the maximum payoff in the repertoire is at least
    twice as high as the mean payoff then we do supposedly
    have hit upon one of the seldom high values.
    Otherwise keep searching by OBSERVING.
    
    In the beginning: OBSERVE if until the repertoire
    contains at least 6 entries.
    
    As usual: Never OBSERVE twice in sequence. 
    """
    if roundsAlive == 0:
        move = OBSERVE
    elif myRepertoire.shape[1] == 0:
        move = INNOVATE
    elif lastAct(myHistory) in [INNOVATE, OBSERVE]:
        move = bestMove(myRepertoire)
    elif myRepertoire.shape[1] < 6:
        move = OBSERVE
    elif repMaxPayoff(myRepertoire) <= 2*repMeanPayoff(myRepertoire):
        move = OBSERVE
    else:
        move = bestMove(myRepertoire)
    return (move, myRepertoire)


def AimingHigh(roundsAlive, myRepertoire, myHistory):
    """'Aiming High' basically plays OBSERVE whenever its last
    payoff was lower than its maximum payoff so far.
    
    But, 1) it never plays OBSERVE if it already played OBSERVE
    or INNOVATE during the previous round.
    2) if the repertoire contains less than five entries it plays
    OBSERVE (under condition of rule 1!)
    3) In 5% of all cases it turns an OBSERVE move into an 
    INNOVATE move
    
    The rationale behind this strategy is that very high payoffs
    are worth searching for; they compensate for long sequences
    of alternating OBSERVE, EXPLOIT moves. 
    
    Fruthermore, it is assumed that once in a while it might be
    worth while to try an innovation.
    """
    if repMaxPayoff(myRepertoire) == 0:
        move = INNOVATE
    elif lastAct(myHistory) in [INNOVATE, OBSERVE]: 
        move = bestMove(myRepertoire)
    elif myRepertoire.shape[1] < 5:
        move = OBSERVE
    else: 
        reference = maxPayoff(myHistory)
        lastPayoff = payoffs(exploits(myHistory))[-1]
        if lastPayoff < reference:
            move = OBSERVE
        else:
            move = bestMove(myRepertoire)
    if move == OBSERVE and rnd() < 0.05:
        move = INNOVATE 
    return (move, myRepertoire) 
        
        
    

def VariateObservations(roundsAlive, myRepertoire, myHistory):
    """Uses a complex scheme of determining whether to OBSERVE:
    
    1. OBSERVE in the first round, INNOVATE in the second round, only
       if OBSERVE did not yield any possible moves in the first round.
    2. OBSERVE in the 2, 5, 9, 14, 20 round unless the repertoire 
       contains at least 10 entries.
    3. OBSERVE with a chance of 75% if the last move yielded less than
       the average payoff so far.
    4. OBSERVE with a chance of 10% if the last move yielded less than
       the maximum payoff so far.
    5. Never OBSERVE two times in sequence.
    
    Rationale: The basic idea is taken from the example strategy: never
    to fall behind one's own the average. However rule 1 and 2 assume
    that one should OBSERVE more often in the beginning, because 
    otherwise the estimate (by one's own average) of how much one can 
    realistically get tends to be too low, because of lack of data.
    
    Other than that it is assumed that there might be more out there
    than only one's own average (rule 4)
    
    Finally, too much time spent observing spoils the profit (75% clause
    in rule 3, rule 5) 
    """
    m,n = myRepertoire.shape
    last = lastAct(myHistory)
    if roundsAlive == 0:
        move = OBSERVE
    elif n == 0: # should almost never happen
        move = INNOVATE
    elif last == OBSERVE or last == INNOVATE:
        move = bestMove(myRepertoire)
    elif roundsAlive in [2, 5, 9, 14, 20] and n < 10:
        move = OBSERVE
    else:
        myMean = meanPayoff(myHistory)
        myMax = maxPayoff(myHistory)
        lastPayoff = payoffs(exploits(myHistory))[-1]
        if lastPayoff < myMean and rnd() < 0.75:
            move = OBSERVE
        elif lastPayoff < myMax and rnd() < 0.1:
            move = OBSERVE
        else:
            move = bestMove(myRepertoire)
    return (move, myRepertoire)
        


def Stuffing(roundsAlive, myRepertoire, myHistory):
    """'Stuffing'-strategy that is only meant to fill the population until
    the 'real' contesters invade the population."""
    if roundsAlive == 0:
        move = INNOVATE
    else:
        move = myRepertoire[0, 0]
    return (move, myRepertoire)


def ASocial(roundsAlive, myRepertoire, myHistory):
    """ASocial never observes. It innovates with a decreasing frequency.
    INNOVATE is played precisely when 'roundsAlive == n*(n+1) / 2' with
    'n' being the size of the repertoire."""
    m, n = myRepertoire.shape
    if roundsAlive == n*(n+1) / 2 and n < ENVIRONMENT_SIZE:
        move = INNOVATE
    else:
        move = bestMove(myRepertoire)    
    return (move, myRepertoire)


def SimpleObserver(roundsAlive, myRepertoire, myHistory):
    """Observes with a decreasing frequency. Whenever 
    'roundsAlive == n*(n+1) / 2' with 'n' being the size of the repertoire,
    OBSERVE is played. INNOVATE is played only if even after the first round
    the repertoire is empty. (In other words: Clear precedence is given
    to observe over innovate.)"""    
    m, n = myRepertoire.shape
    if roundsAlive == n*(n+1) / 2 and n < ENVIRONMENT_SIZE:
        move = OBSERVE
    elif n == 0: # play INNOVATE only if there was nothing to observe
        move = INNOVATE
    else:
        move = bestMove(myRepertoire)      
    return (move, myRepertoire)
       

def Childhood(roundsAlive, myRepertoire, myHistory):
    """idea by Martin Neuman, implemented by Eckhart Arnold

    The strategy Childhood observes more often during the early rounds of 
    its life time to fill the repertoire and seldom afterwards.
    
    The strategy in detail:
    
    - Play OBSERVE in the first, third and seventh round.
    - Play OBSERVE every 15th round.
    - Turn an OBSERVE move into an INNOVATE move with a probability of 1%    
    - Play INNOVATE if the repertoire is still empty after the first round.
    - In all othter cases chose the best EXPLOIT move from the repertoire.
    """
    if roundsAlive in [0, 2, 6]:
        move = OBSERVE
    elif myRepertoire.shape[1] == 0:
        move = INNOVATE
    elif roundsAlive % 15 == 0 or repMaxPayoff(myRepertoire) == 0:
        move = OBSERVE
    else:
        move = bestMove(myRepertoire)
    if move == OBSERVE and rnd() < 0.01: move = INNOVATE
    return (move, myRepertoire)   



    