"""cultaptation engine: Toolkit

Contains some useful helper functions for the cultaptation engine 
and strategies. 

Copyright (c) 2008 by Eckhart Arnold (www.eckhartarnold.de)
                   
License: GNU Public License V3
(http://www.gnu.de/documents/gpl-3.0.en.html)

This program is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the 
Free Software Foundation, either version 3 of the License, or (at your 
option) any later version.

This program is distributed in the hope that it will be useful, but 
WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
Public License for more details.

You should have received a copy of the GNU General Public License along 
with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from random import random as rnd
from numpy import array, argsort, where, mean, ravel, squeeze, concatenate, \
                  zeros, int16, int32, float32

from Configuration import *


def RandomSelect(distribution):
    """Randomly selects an item from a distribution. A distribution
    is a list or a tuple of real numbers form 0 to 1 that add up to 1.
    The items are the indices of the real numbers in the list. The
    real numbers are considered probability values with which the 
    corresponding item is selected. 
    """
    def rselect(slices, r, a, b):
        if b-a <= 1: return a
        m = (a+b)/2
        if r >= slices[m]:  
            return rselect(slices, r, m, b)
        else: 
            return rselect(slices, r, a, m)

    if len(distribution) == 0:
        raise ValueError,"Cannot select an element from an empty distribution"
    slices = [0.0]
    for p in distribution:  
        slices.append(p+slices[-1])
    return rselect(slices, rnd(), 0, len(slices)) 


def pcvariate(minValue, maxValue):
    """A function that variates the pc value (rate of change of environment)
    for the melee match. It is geared towards returning values close to 
    'minValue' more often.
    """
    # let it be more likely that the value is small
    return minValue + rnd()**3 * (maxValue-minValue)
    # uniform distribution
    #return uniform(minValue, maxValue)

###############################################################################
#
# functions that help selecting columns of from the history table
#
###############################################################################

def size(matrix):
    """-> The size (i.e. number of columns) of the history table or the
    repertoire."""
    return matrix.shape[1]

def nri(history):
    """-> array of indices that mark a new round in the history table. Thus, 
    for OBSERVE moves that occupy several columns in the history table for one
    and the same round (because more than one other individual was observed)
    only the index of the first column will be contained in the array.
    """
    indices = where(history[0,:-1] != history[0,1:])[0]+1
    return concatenate((array([0]), indices))

def fold(history):
    """-> a table where only one column occurs for every new round. In
    case there are several entries for one round (as can happen with
    OBSERVE acts) only the first entry is contained in the returned
    array. Thus, fold(allObserves(history)) returns only the first
    column of each OBSERVE act. 
    """
    return history[:,nri(history)]

def splitHistory(history):
    raise NotImplementedError

def exploits(history):
    """-> all EXPLOIT entries from the history table."""
    return history[:, where(history[1,:] >= EXPLOIT_MIN)]

def innovates(history):
    """-> all INNOVATE entries from the history table."""
    return history[:, where(history[1,:] == INNOVATE)]

def allObserves(history):
    """-> all OBSERVE entries from the history table."""
    return history[:, where(history[1,:] == OBSERVE)]

def foldedObserves(history):
    """-> all OBSERVE entries from the history table, but for entires that
    occupy several columns only the first column is taken."""
    return fold(allObserves(history))

def lastNActs(history, n):
    """-> a list of the last 'n' acts. If the history is shorter
    than 'n' the size of the tuple will be smaller accordingly."""
    if history.shape[1] > 0:
        return history[1, nri(history)[-n:]]
    else: return array([ILLEGAL], dtype=int16)

def lastAct(history):
    """Returns the type of act of the last move, i.e. INNOVATE, 
    OBSERVE or EXPLOIT or ILLEGAL if the history is empty."""
    if history.shape[1] > 0:  return history[1,-1]
    else: return ILLEGAL

def firstNActs(history, n):
    """-> a list of the first 'n' acts. If the history is shorter
    than 'n' the size of the tuple will be smaller accordingly."""
    if history.shape[1] > 0:
        return history[1, nri(history)[:n]]
    else: return array([ILLEGAL], dtype=int16)

def tail(history, n):
    """Returns the tail part of the history table that contains at most
    the 'n' latest rounds."""
    if history.shape[1] < n: return history
    t = nri(history)[-n:][0]
    return history[:, t:]

def head(history, n):
    """Resturns the head part of the history table containing (at most)
    the 'n' first rounds.
    """
    if history.shape[1] < n: return history
    t = nri(history)[:n][0]
    return history[:, t:]

def meanPayoff(history, n=0):
    """-> the mean payoff from the last 'n' EXPLOIT moves. if 'n'=0 then all
    previous moves are considered."""
    if history.shape[1] == 0: return 0
    ex = where(history[1,:] >= EXPLOIT_MIN)[0]
    return mean(history[3, ex[-n:]])

def maxPayoff(history, n=0):
    """-> the max payoff from the last 'n' EXPLOIT moves. if 'n'=0 then all
    previous moves are considered."""
    if history.shape[1] == 0: return 0
    ex = where(history[1,:] >= EXPLOIT_MIN)[0]
    return max(history[3, ex[-n:]])

###############################################################################
#
# functions that help selecting rows from the (potentially reduced) history
# table
#
###############################################################################

def rounds(table):
    """-> all round numbers for which the 'table' has an entry,
    i.e. rounds(exploits(history)) gives the rounds when EXPLOIT
    acts occured in the history."""
    return ravel(table[0,:])

def acts(table):
    """-> the array of acts for which 'table' has an entry,
    i.e. acts(fold(history)) delivers the array of acts 
    played so far with one entry for each round."""
    return ravel(table[1,:])

def cells(table):
    """-> the cells of the environment which have been accessed
    (either to EXPLOIT or by gathering information through INNOVATE or OBSERVE
    acts), i.e. cells(innovates(history)) returns all cells that were
    examined through INNOVATE acts."""
    return ravel(table[2:])

def payoffs(table):
    """-> the payoffs associated with the entries in the 'table', i.e.
    payoffs(observes(history)) returns all (potentially mistaken) payoffs that
    became known through OBSERVE acts."""
    return ravel(table[3:]) 
        
#def genRepertoire(history):
#    """-> repertoire, freshly reconstructed from the history table. This can be
#    used to create repertoires that do not contain any stale entries, by passing
#    only the last n rounds of the history tabe."""
#    repertoire = zeros((2, ENVIRONMENT_SIZE), dtype = int16)
#    repSize = 0
#    for i in xrange(history.shape[1]):
#        move = history[2, i]
#        payoff = history[3, i]
#        if move >= EXPLOIT_MIN and not move in repertoire[0,:repSize]:
#            repertoire[:,repSize] = [move, payoff]
#            repSize += 1
#        else:
#            repertoire[1, where(repertoire[0,:]==move)] = payoff
#    return repertoire[:, :repSize]

def genRepertoire(history):
    """-> repertoire, freshly reconstructed from the history table. This can be
    used to create repertoires that do not contain any stale entries, by passing
    only the last n rounds of the history tabe."""
    repertoire = zeros((2, ENVIRONMENT_SIZE), dtype = int16)
    repSize = 0
    for i in xrange(history.shape[1]-1, -1, -1):
        move = history[2, i]
        if move >= EXPLOIT_MIN and not move in repertoire[0,:repSize]:
            payoff = history[3, i]
            repertoire[:,repSize] = [move, payoff]
            repSize += 1
    return repertoire[:, :repSize]
                
#def dynamicRepertoire(history):
#    """-> reconstructed repertoire, where the payoff values are being decresed,
#    the older the information is!"""
#    repertoire = zeros((2, ENVIRONMENT_SIZE), dtype = int16)
#    repSize = 0
#    for i in xrange(history.shape[1]):
#        move = history[2, i]
#        age = history[0, -1] - history[0, i] + int(history[1, i] == OBSERVE)
#        factor = 0.95 ** age
#        payoff = int16(history[3, i] * factor + 0.5) 
#        if move >= EXPLOIT_MIN and not move in repertoire[0,:repSize]:
#            repertoire[:,repSize] = [move, payoff]
#            repSize += 1
#        else:
#            repertoire[1, where(repertoire[0,:repSize]==move)] = payoff
#    return repertoire[:,:repSize]

def dynamicRepertoire(history):
    """-> reconstructed repertoire, where the payoff values are being decresed,
    the older the information is!"""
    repertoire = zeros((2, ENVIRONMENT_SIZE), dtype = int16)
    repSize = 0
    for i in xrange(history.shape[1]-1, -1, -1):
        move = history[2, i]
        if move >= EXPLOIT_MIN and move not in repertoire[0,:repSize]:
            age = history[0, -1] - history[0, i] + int(history[1, i] == OBSERVE)
            payoff = int16(history[3, i] * (0.95**age) + 0.5) 
            repertoire[:,repSize] = [move, payoff]
            repSize += 1
    return repertoire[:,:repSize]
    
###############################################################################
#
# functions for accessing the repertoire
#
###############################################################################

def bestMove(repertoire):
    """-> the move that yields the highest (expected) payoff. If the 
    repertoire is empty, ILLEGAL is returned."""
    if repertoire.shape[1] == 0:  return ILLEGAL
    rankedR_Matrix = repertoire[:,argsort(repertoire[1,:])]
    return rankedR_Matrix[0,-1]


def repMaxPayoff(repertoire):
    """-> the (expected) payoff associated with the best move in the 
    repertoire. If the repertoire is empty, 0 is returned."""
    if repertoire.shape[1] == 0: return 0
    return max(repertoire[1,:])
#    rankedR_Matrix = repertoire[:,argsort(repertoire[1,:])]
#    return rankedR_Matrix[1,-1]

def repMeanPayoff(repertoire):
    """-> the mean (expected) payoff over all moves in the repertoire.
    If the repertoire is empty, 0 is returned."""
    if repertoire.shape[1] == 0: return 0
    return mean(repertoire[1,:])

def isEmpty(repertoire):
    """-> True, if the repertoire does not contain any moves at all."""
    return repertoire.shape[1] == 0 
    
def decayRepertoire(repertoire):
    """Decrease every entry in the repertoire by 1. A possible use of
    this function is to simulate the decaying value of the information about 
    payoffs over time, because fresh payoff information is always overwritten
    in the repertoire."""
    repertoire[1,] -= 1
    repertoire[1, where(repertoire[1,:] < 0)] = 0
